# This file allows to compute the objects CV3.RData, CV5.RData, CV10.RData, MCCV2.3, MCCV4.5, MCCV9.10, LOOCV and BOOT and to store them in the directory "results".
# It necessitates to have the data files (adrenal_dahia.RData, bladder_blaveri.RData,...) stored in the directory "data_R" using the code datahandling.R.
# Figures based on the objects CV3.RData, CV5.RData, CV10.RData, MCCV2.3, MCCV4.5, MCCV9.10, LOOCV and BOOT and presented in the paper can be reproduced using code from the file analysis_comparison_paper.R.



library(CMA)

# Define function to perform repeated subsampling analyses
MCCV_pls<-function(ratio,niter,datasetnames,methodnames,mainname)
{
  MCCV<-array(NA,dim=c(niter,length(datasetnames),length(methodnames)))
  dimnames(MCCV)<-list(paste("iter",1:niter),datasetnames,methodnames)
  for (i in 1:length(datasetnames))
  {
    print(i)
    datasetname<-datasetnames[i]
    load(paste("../data_R/",datasetname,".RData",sep=""))
    X<-dataset$X
    Y<-dataset$Y
    if (nlevels(Y)==2) # we just consider two class problems here
    { set.seed(1011) # ! reproducibility
      learn<- GenerateLearningsets(y=Y,method="MCCV",niter=niter,ntrain=round(length(Y)*ratio))
      varsel<-GeneSelection(X=X,y=Y,learningsets=learn,method="t.test")
          
      plsldanbgene1000comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=100,classifier=pls_ldaCMA))

      plsldanbgene1000comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=100,classifier=pls_ldaCMA))

      plsldanbgene1000comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=100,classifier=pls_ldaCMA))

      
      MCCV[,i,]<-cbind(plsldanbgene1000comp1@score,plsldanbgene500comp1@score,plsldanbgene200comp1@score,plsldanbgene100comp1@score,
              plsldanbgene1000comp2@score,plsldanbgene500comp2@score,plsldanbgene200comp2@score,plsldanbgene100comp2@score,
              plsldanbgene1000comp3@score,plsldanbgene500comp3@score,plsldanbgene200comp3@score,plsldanbgene100comp3@score)
    }
  }
  save(MCCV,file=paste("results/MCCV",mainname,".RData",sep=""))
 }



# Define function to perform CV analyses
CV_pls<-function(ncv,fold,datasetnames,methodnames,mainname)
{
  CV<-array(NA,dim=c(ncv*fold,length(datasetnames),length(methodnames)))
  dimnames(CV)<-list(paste("iter",1:(ncv*fold)),datasetnames,methodnames)
  for (i in 1:length(datasetnames))
  {
    print(i)
    datasetname<-datasetnames[i]
    load(paste("../data_R/",datasetname,".RData",sep=""))
    X<-dataset$X
    Y<-dataset$Y
    if (nlevels(Y)==2) # we just consider two class problems here
    { set.seed(1011) # ! reproducibility
      learn<- GenerateLearningsets(y=Y,method="CV",niter=ncv,fold=fold)
      varsel<-GeneSelection(X=X,y=Y,learningsets=learn,method="t.test")
          
      plsldanbgene1000comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=100,classifier=pls_ldaCMA))

      plsldanbgene1000comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=100,classifier=pls_ldaCMA))

      plsldanbgene1000comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=100,classifier=pls_ldaCMA))

      CV[,i,]<-cbind(plsldanbgene1000comp1@score,plsldanbgene500comp1@score,plsldanbgene200comp1@score,plsldanbgene100comp1@score,
              plsldanbgene1000comp2@score,plsldanbgene500comp2@score,plsldanbgene200comp2@score,plsldanbgene100comp2@score,
              plsldanbgene1000comp3@score,plsldanbgene500comp3@score,plsldanbgene200comp3@score,plsldanbgene100comp3@score)
    }
  }
  save(CV,file=paste("results/CV",mainname,".RData",sep=""))
}


# Define function to perform bootstrap analyses
BOOT_pls<-function(niter,datasetnames,methodnames,mainname="")
{
  BOOT<-array(NA,dim=c(niter,length(datasetnames),length(methodnames)))
  dimnames(BOOT)<-list(paste("iter",1:niter),datasetnames,methodnames)
  for (i in 1:length(datasetnames))
  {
    print(i)
    datasetname<-datasetnames[i]
    load(paste("../data_R/",datasetname,".RData",sep=""))
    X<-dataset$X
    Y<-dataset$Y
    if (nlevels(Y)==2) # we just consider two class problems here
    { set.seed(1011) # ! reproducibility
      learn<- GenerateLearningsets(y=Y,method="bootstrap",niter=niter)
      varsel<-GeneSelection(X=X,y=Y,learningsets=learn,method="t.test")
          
      plsldanbgene1000comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=100,classifier=pls_ldaCMA))

      plsldanbgene1000comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=100,classifier=pls_ldaCMA))

      plsldanbgene1000comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=100,classifier=pls_ldaCMA))

      
      BOOT[,i,]<-cbind(plsldanbgene1000comp1@score,plsldanbgene500comp1@score,plsldanbgene200comp1@score,plsldanbgene100comp1@score,
              plsldanbgene1000comp2@score,plsldanbgene500comp2@score,plsldanbgene200comp2@score,plsldanbgene100comp2@score,
              plsldanbgene1000comp3@score,plsldanbgene500comp3@score,plsldanbgene200comp3@score,plsldanbgene100comp3@score)
    }
  }
  save(BOOT,file=paste("results/BOOT",mainname,".RData",sep=""))
 }


# Define function to perform LOOCV analyses
LOOCV_pls<-function(datasetnames,methodnames,mainname="")
{
  LOOCV<-array(NA,dim=c(length(datasetnames),length(methodnames)))
  dimnames(LOOCV)<-list(datasetnames,methodnames)
  for (i in 1:length(datasetnames))
  {
    print(i)
    datasetname<-datasetnames[i]
    load(paste("../data_R/",datasetname,".RData",sep=""))
    X<-dataset$X
    Y<-dataset$Y
    if (nlevels(Y)==2) # we just consider two class problems here
    { set.seed(1011) # ! reproducibility
      learn<- GenerateLearningsets(y=Y,method="LOOCV")
      varsel<-GeneSelection(X=X,y=Y,learningsets=learn,method="t.test")
          
      plsldanbgene1000comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp1<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=1,nbgene=100,classifier=pls_ldaCMA))

      plsldanbgene1000comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp2<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=2,nbgene=100,classifier=pls_ldaCMA))

      plsldanbgene1000comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=1000,classifier=pls_ldaCMA))
      plsldanbgene500comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=500,classifier=pls_ldaCMA))
      plsldanbgene200comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=200,classifier=pls_ldaCMA))
      plsldanbgene100comp3<-evaluation(classification(X=X,y=Y,learningsets=learn,genesel=varsel,comp=3,nbgene=100,classifier=pls_ldaCMA))

      LOOCV[i,]<-c(mean(plsldanbgene1000comp1@score),mean(plsldanbgene500comp1@score),mean(plsldanbgene200comp1@score),mean(plsldanbgene100comp1@score),
              mean(plsldanbgene1000comp2@score),mean(plsldanbgene500comp2@score),mean(plsldanbgene200comp2@score),mean(plsldanbgene100comp2@score),
              mean(plsldanbgene1000comp3@score),mean(plsldanbgene500comp3@score),mean(plsldanbgene200comp3@score),mean(plsldanbgene100comp3@score))
    }
  }
  
  save(LOOCV,file=paste("results/LOOCV",mainname,".RData",sep=""))
 }



datasetnames<-c("adrenal_dahia","bladder_blaveri","bladder_dyrskjot",
                  "bladder_sanchez-carbayo","breast_desmedt","breast_farmer","breast_gruvberger","breast_kreike",
                  "breast_ma_2","breast_minn","breast_perou","breast_sharma","breast_sotiriou","breast_veer",
                  "breast_wang","breast_west","cervical_wong","cns_pomeroy_2","colon_alon","colon_laiho",
                  "colon_lin_1","colon_watanabe","gastric_hippo","glioma_freije","glioma_nutt","glioma_phillips",
                  "glioma_rickman","head_neck_chung","headneck_pyeon_2","leukemia_armstrong","leukemia_bullinger_2",
                  "leukemia_golub","leukemia_gutierrez","leukemia_haslinger","leukemia_wei","leukemia_yagi",
                  "liver_chen","liver_iizuka","liver_ye","lung_barret","lung_beer","lung_bhattacharjee_2","lung_bild",
                  "lung_wigle","lymphoma_alizadeh","lymphoma_booman","lymphoma_rosenwald","lymphoma_shipp",
                  "medulloblastoma_macdonald","melanoma_talantov","mixed_chowdary","mixed_ramaswamy","myeloma_tian",
                  "oral_odonnell","ovarian_gilks","ovarian_jazaeri_3","ovarian_li_and_campbell","ovarian_schwartz",
                  "pancreas_ishikawa","prostate_singh","prostate_tomlins","prostate_true_2","renal_williams",
                  "sarcoma_detwiller","srbct_khan")
                
methodnames<-c("plsldanbgene1000comp1","plsldanbgene500comp1","plsldanbgene200comp1","plsldanbgene100comp1",
              "plsldanbgene1000comp2","plsldanbgene500comp2","plsldanbgene200comp2","plsldanbgene100comp2",
              "plsldanbgene1000comp3","plsldanbgene500comp3","plsldanbgene200comp3","plsldanbgene100comp3")

CV_pls(ncv=2,fold=10,datasetnames,methodnames,mainname="10")
CV_pls(ncv=2,fold=5,datasetnames,methodnames,mainname="5")
CV_pls(ncv=2,fold=3,datasetnames,methodnames,mainname="3")

MCCV_pls(niter=2,ratio=2/3,datasetnames,methodnames,mainname="2.3")
MCCV_pls(niter=2,ratio=4/5,datasetnames,methodnames,mainname="4.5")
MCCV_pls(niter=2,ratio=9/10,datasetnames,methodnames,mainname="9.10")

LOOCV_pls(datasetnames,methodnames,mainname="")
BOOT_pls(niter=1000,datasetnames,methodnames,mainname="")



#CV_pls(ncv=100,fold=10,datasetnames,methodnames,mainname="10")
#CV_pls(ncv=200,fold=5,datasetnames,methodnames,mainname="5")
#CV_pls(ncv=333,fold=3,datasetnames,methodnames,mainname="3")

#MCCV_pls(niter=1000,ratio=2/3,datasetnames,methodnames,mainname="2.3")
#MCCV_pls(niter=1000,ratio=4/5,datasetnames,methodnames,mainname="4.5")
#MCCV_pls(niter=1000,ratio=9/10,datasetnames,methodnames,mainname="9.10")

#LOOCV_pls(datasetnames,methodnames,mainname="")
#BOOT_pls(niter=1000,datasetnames,methodnames,mainname="")

