# This file allows to reproduce the figures presented in the paper.
# It necessitates that the objects CV3.RData, CV5.RData, CV10.RData, MCCV2.3, MCCV4.5, MCCV9.10, LOOCV and BOOT are stored in the directory "results".

####################
####################
##### Figure 1 #####
####################
####################

pdf("lines.pdf",height=5,width=5)
load("results/CV5.RData")
x4.5<-apply(CV,MARGIN=3,FUN=mean,na.rm=TRUE)
load("results/CV10.RData")
x9.10<-apply(CV,MARGIN=3,FUN=mean,na.rm=TRUE)
load("results/CV3.RData")
x2.3<-apply(CV,MARGIN=3,FUN=mean,na.rm=TRUE)

plot(c(3,5,10),c(x2.3[1],x4.5[1],x9.10[1]),type="b",pch=1,ylab="mean error rate",xlab="",xlim=c(2,16),ylim=c(0.19,0.27),xaxt="n")
for (i in 2:12)
 {
 points(c(3,5,10),c(x2.3[i],x4.5[i],x9.10[i]),type="b",pch=i,lty=1)
 }
text(x=3,y=0.195,label="3-fold")
text(x=5,y=0.195,label="5-fold")
text(x=10,y=0.195,label="10-fold")
legend(x=11,y=0.27,c("c=1, p*=1000","c=1, p*=500","c=1, p*=200","c=1, p*=100",
			"c=2, p*=1000","c=2, p*=500","c=2, p*=200","c=2, p*=100",
			"c=3, p*=1000","c=3, p*=500","c=3, p*=200","c=3, p*=100"),pch=1:12)
dev.off()


pvalmat<-function(results)
{
pvalmat<-matrix(NA,ncol(results),ncol(results))
for (i in 2:ncol(results))
 {
 for (j in 1:(i-1))
  {
  pvalmat[i,j]<-t.test(results[,i]-results[,j])$p.value
  }
 }
return(pvalmat)
}


pdf("hist_pval.pdf",width=5,height=5)
load("results/CV3.RData")
my.pval<-pvalmat(apply(CV[1:999,,],MARGIN=c(2,3),FUN=mean))
colnames(my.pval)<-dimnames(CV)[[3]]
rownames(my.pval)<-dimnames(CV)[[3]]
hist(my.pval,breaks=20,xlab="p-value",ylab="frequency",main="")
dev.off()

####################
####################
##### Figure 2 #####
####################
####################


meanvec<-function(results)
{
meanvector<-c()
for (i in 2:ncol(results))
 {
 for (j in 1:(i-1))
  {
  meanvector<-c(meanvector,abs(mean(results[,i]-results[,j],na.rm=TRUE)))
  }
 }
return(meanvector)
}


pdf("mean.pdf",width=7,height=10)
par(las=2,mfrow=c(3,1))
load("results/MCCV2.3.RData")
load("results/CV3.RData")
load("results/BOOT.RData")
load("results/LOOCV.RData")
boxplot(meanvec(apply(MCCV[1:3,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:24,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:99,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:249,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:501,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:999,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:3,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:24,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:99,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:249,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:501,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:999,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(BOOT[1:3,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(BOOT[1:24,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(BOOT[1:99,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(BOOT[1:249,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(BOOT[1:501,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(BOOT[1:999,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(LOOCV),at=c(0:5,7:12,14:19,21),xlim=c(0,21),
	names=c("SUB3","SUB24","SUB99","SUB249","SUB501","SUB999","CV1","CV8","CV33","CV83","CV167","CV333","BOOT3","BOOT24","BOOT99","BOOT249","BOOT501","BOOT999","LOOCV"),
	main="2:1 subsampling, 3-fold-CV, BOOT, LOOCV", ylab="mean",ylim=c(-0.01,0.07))
abline(v=6)
abline(v=13)
abline(v=20)


load("results/MCCV4.5.RData")
load("results/CV5.RData")
load("results/BOOT.RData")
load("results/LOOCV.RData")
boxplot(meanvec(apply(MCCV[1:5,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:25,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:100,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:250,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:500,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:1000,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:5,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:25,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:100,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:250,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:500,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:1000,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(LOOCV),at=c(0:5,7:12,14),xlim=c(0,14),
	names=c("SUB5","SUB25","SUB100","SUB250","SUB500","SUB1000","CV1","CV5","CV20","CV50","CV100","CV200","LOOCV"),
	main="4:1 subsampling, 5-fold-CV,  LOOCV", ylab="mean",ylim=c(-0.01,0.07))
abline(v=6)
abline(v=13)


load("results/MCCV9.10.RData")
load("results/CV10.RData")
load("results/BOOT.RData")
load("results/LOOCV.RData")
boxplot(meanvec(apply(MCCV[1:10,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:50,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:200,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:500,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(MCCV[1:1000,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:10,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:50,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:200,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:500,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(apply(CV[1:1000,,],MARGIN=c(2,3),FUN=mean)),
	meanvec(LOOCV),at=c(0:4,6:10,12),xlim=c(0,12),
	names=c("SUB10","SUB50","SUB200","SUB500","SUB1000","CV1","CV5","CV20","CV50","CV100","LOOCV"),
	main="9:1 subsampling, 10-fold-CV,  LOOCV", ylab="mean",ylim=c(-0.01,0.07))
abline(v=5)
abline(v=11)
dev.off()



####################
####################
##### Figure 3 #####
####################
####################

sdvec<-function(results)
{
sdvector<-c()
for (i in 2:ncol(results))
 {
 for (j in 1:(i-1))
  {
  sdvector<-c(sdvector,sd(results[,i]-results[,j],na.rm=TRUE))
  }
 }
return(sdvector)
}


pdf("sd.pdf",width=7,height=10)
par(las=2,mfrow=c(3,1),mar=c(5,4,4,5)+0.1,cex.lab=1.5)
load("results/MCCV2.3.RData")
load("results/CV3.RData")
load("results/BOOT.RData")
load("results/LOOCV.RData")
boxplot(sdvec(apply(MCCV[1:3,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:24,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:99,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:249,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:501,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:999,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:3,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:24,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:99,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:249,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:501,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:999,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(BOOT[1:3,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(BOOT[1:24,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(BOOT[1:99,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(BOOT[1:249,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(BOOT[1:501,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(BOOT[1:999,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(LOOCV),at=c(0:5,7:12,14:19,21),xlim=c(0,21),
	names=c("SUB3","SUB24","SUB99","SUB249","SUB501","SUB999","CV1","CV8","CV33","CV83","CV167","CV333","BOOT3","BOOT24","BOOT99","BOOT249","BOOT501","BOOT999","LOOCV"),
	main="2:1 subsampling, 3-fold-CV, BOOT, LOOCV", ylab="",ylim=c(0,0.15))
# draw y-axis on the right side
ticksvec<-numeric(5)
ticksvec[1]<-power.t.test(n=3,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[2]<-power.t.test(n=5,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[3]<-power.t.test(n=10,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[4]<-power.t.test(n=20,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[5]<-power.t.test(n=40,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
axis(4,at=ticksvec,labels=c(3,5,10,20,40))
mtext("J",side=4,line=3)
mtext(expression(paste(sigma)),side=2,line=3)
abline(v=6)
abline(v=13)
abline(v=20)

load("results/MCCV4.5.RData")
load("results/CV5.RData")
load("results/BOOT.RData")
load("results/LOOCV.RData")
boxplot(sdvec(apply(MCCV[1:5,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:25,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:100,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:250,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:500,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:1000,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:5,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:25,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:100,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:250,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:500,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:1000,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(LOOCV),at=c(0:5,7:12,14),xlim=c(0,14),
	names=c("SUB5","SUB25","SUB100","SUB250","SUB500","SUB1000","CV1","CV5","CV20","CV50","CV100","CV200","LOOCV"),
	main="4:1 subsampling, 5-fold-CV,  LOOCV", ylab="",ylim=c(0,0.15))
# draw y-axis on the right side
ticksvec<-numeric(5)
ticksvec[1]<-power.t.test(n=3,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[2]<-power.t.test(n=5,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[3]<-power.t.test(n=10,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[4]<-power.t.test(n=20,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[5]<-power.t.test(n=40,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
axis(4,at=ticksvec,labels=c(3,5,10,20,40))
mtext("J",side=4,line=3)
mtext(expression(paste(sigma)),side=2,line=3)
abline(v=6)
abline(v=13)


load("results/MCCV9.10.RData")
load("results/CV10.RData")
load("results/BOOT.RData")
load("results/LOOCV.RData")
boxplot(sdvec(apply(MCCV[1:10,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:50,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:200,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:500,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(MCCV[1:1000,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:10,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:50,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:200,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:500,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(apply(CV[1:1000,,],MARGIN=c(2,3),FUN=mean)),
	sdvec(LOOCV),at=c(0:4,6:10,12),xlim=c(0,12),
	names=c("SUB10","SUB50","SUB200","SUB500","SUB1000","CV1","CV5","CV20","CV50","CV100","LOOCV"),
	main="9:1 subsampling, 10-fold-CV,  LOOCV", ylab="",ylim=c(0,0.15))
abline(v=5)
abline(v=11)
# draw y-axis on the right side
ticksvec<-numeric(5)
ticksvec[1]<-power.t.test(n=3,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[2]<-power.t.test(n=5,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[3]<-power.t.test(n=10,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[4]<-power.t.test(n=20,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
ticksvec[5]<-power.t.test(n=40,delta=0.05,sd=NULL,power=0.8,type="paired",alternative="one.sided")$sd
axis(4,at=ticksvec,labels=c(3,5,10,20,40))
mtext("J",side=4,line=3)
mtext(expression(paste(sigma)),side=2,line=3)
dev.off()



